#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    Script d'ajout de groupe dans la base Posh
"""
import getopt, sys
from posh.api.scribe_api import ScribePosh
DOC = """
Usage : add_group.py [OPTION] [GROUPNAME]
Synchronise un groupe et ses utilisateurs auprès de la base de données de Envole

-t,  --type       : Le type de groupe à rajouter dans la base (1:usergroup, 2:profilegroup)
-p,  --parent     : Id d'un éventuel groupe parent

Ex :
1) add_group.py -t 1 mongroupe1
rajoute le groupe 'mongroupe1' comme groupe 'carnet'

Not implemented yet :
2) add_group.py -t 2 -p 1 mongroupe2
rajoute le groupe 'mongroupe2' comme groupe de profile enfant du groupe d'id '1'
"""

def usage():
    """
        print la doc
    """
    print DOC

def main():
    """
        lance la commande
    """
    try:
        opts, args = getopt.getopt(sys.argv[1:], "t:p:h", ["type", "parent", "help"])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)
    try:
        groupname = args[0]
    except IndexError:
        print("-- GROUP NAME IS MISSING --")
        usage()
        sys.exit(2)
    grouptype = '1'
    parentid = 0 # (c'est la valeur par défaut pour l'écriture dans la base)
    for option, value in opts:
        if option in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif option in ("-t", "--type"):
            grouptype = value.strip()
            if grouptype not in ('1', '2'):
                print("Group type should be 1 (notebook) or 2 (profile)")
                usage()
                sys.exit(2)
            if grouptype == '1':
                break
        elif option in ("-p", "--parent"):
            try:
                assert int(value) >= 0
                parentid = value
            except AssertionError:
                print "Parent id should be an integer >=0 "
                usage()
                sys.exit(2)
            except ValueError:
                print "Parent id should be an integer"
                usage()
                sys.exit(2)
        else:
            usage()
            assert False, "unhandled option"
    api = ScribePosh()
    if grouptype == '1':
        api.synchronize_usergroup(groupname)
    else:
        api.synchronize_group(groupname)
    sys.exit(0)

if __name__ == "__main__":
    main()

