#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    Synchronise l'annuaire ldap avec le réseau social posh
"""
import getopt, sys
from posh.api.scribe_api import EnvoleUserGroups
DOC = """
Usage : synchronize_usergroups.py

Synchronise les classes de l'annuaire ldap dans le réseau social Posh

Optionnels:
-t,  --type   : type de groupe à synchroniser (Classe, Equipe, Service...)
-n,  --name   : nom du groupe à synchroniser
-f,  --force  : forcer la synchronisation du groupe (dans le cas où un groupe a déjà été créé il n'est pas resynchronisé sans cette option)

Ex :
1) synchronize_usergroups.py
ajoute toutes les classes et utilisateurs de l'annuaire dans la base portail (inscrit les enseignants aux classes)

2) synchronize_usergroups.py -t Niveau
ajoute les niveau et leurs utilisateurs

3) synchronize_usergroups.py -n 3eme1
ajoute le groupe 3eme1 et ses utilisateurs
"""

def usage():
    """
        print la doc
    """
    print DOC

def main():
    """
        lance la commande
    """
    try:
        opts, args = getopt.getopt(sys.argv[1:], "n:t:fh",
                         ["name", "type", "force", "help"])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    grouptype = None
    groupname = None
    force = False
    for option, value in opts:
        if option in ("-h", "--help"):
            usage()
            sys.exit(0)
        elif option in ("-t", "--type"):
            grouptype = value
        elif option in ("-n", "--name"):
            groupname = value
        elif option in ("-f", "--force"):
            force = True
        else:
            usage()
            assert False, "unhandled option"
    manager = EnvoleUserGroups()
    if groupname:
        manager.synchronize_usergroup(groupname, force=force)
    elif grouptype:
        manager.synchronize_usergroups(_type=grouptype, force=force)
    else:
        manager.synchronize_usergroups(_type='Classe', force=force)
    sys.exit(0)

if __name__ == "__main__":
    main()
