#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# group_api.py
#
# Gestion des groupes profil (admin)
###########################################################################
"""
    Création/modification/suppression des groupes Posh
    from group_api import PoshGroupApi
    gr = PoshGroupApi()
    gr.add_group('groupetest')

"""
from os.path import join

from posh.logger.logger import getLogger
from posh.config.api import POSH_API_URL, ACTION_DICT

from base_api import PoshBaseApi
from request import PoshRequest
from xml_parse import XmlParse

log = getLogger("api.group_api")

class PoshGroupApi(PoshBaseApi):
    """
        Gestionnaire de groupe du portail Posh
    """
    def add_group(self, groupname, group_parent_id=0):
        """
            Ajoute un groupe Posh
            groupname:nom du groupe
            group_parent_id:id d'un éventuel groupe parent
        """
        log.debug("# Ajout du groupe %s #" % (groupname,))
        script_url = join(POSH_API_URL, ACTION_DICT['add_group'])
        post_args = dict(groupname=groupname,
                         groupid=group_parent_id,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        if xml.success():
            log.debug(" + Le groupe %s a été inséré dans la \
base." % (groupname,))
        else:
            log.debug(" - Le groupe %s existe déjà dans la \
base" % (groupname,))
        group_id = xml.groupid()
        return group_id

    def get_groupid(self, groupname, group_parent_id=0):
        """
            Renvoie l'id d'un groupe
            groupname:nom du groupe
            group_parent_id:id d'un éventuel groupe parent
        """
        log.debug("    # Récupération de l'id du groupe %s #" % (groupname,))
        script_url = join(POSH_API_URL, ACTION_DICT['get_groupid'])
        post_args = dict(groupname=groupname,
                         parent_id=group_parent_id,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        group_id = xml.groupid()
        return group_id

    def del_group(self, groupname, group_parent_id=0):
        """
            Supprime le groupe posh
            groupname:nom du groupe
            group_parent_id:id d'un éventuel groupe parent
        """
        log.debug("# Suppression du groupe d'id :%s #" %(groupname,))
        groupid = self.get_groupid(groupname, group_parent_id)
        script_url = join(POSH_API_URL, ACTION_DICT['del_group'])
        post_args = dict(groupid=groupid,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.success()

    def mod_group(self, groupname, group_parent_id, newname):
        """
            modifie le nom d'un groupe
            groupid: id du groupe
            newname: nouveau nom du groupe
        """
        log.debug("# Modification du nom du groupe : %s #" % (groupname,))
        groupid = self.get_groupid(groupname, group_parent_id)
        script_url = join(POSH_API_URL, ACTION_DICT['mod_group'])
        post_args = dict(groupid=groupid,
                         groupname=newname,
                         key=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.success()
