#-*-coding:utf-8-*-
"""
    outil d'éxécution de requête http
    >>> from posh.api.request import PoshRequest
    >>> a = PoshRequest(url, post_args)
    >>> result = a.request()
"""
from os.path import isfile
from urllib import urlencode
import urllib2
import cookielib

from posh.config.api import COOKIE
from posh.logger.logger import getLogger
from posh.error import RequestError

log = getLogger("api.request")

def get_cookie_handler():
    """
        Renvoie le gestionnaire de cookie
    """
    cookie_handler = cookielib.LWPCookieJar()
    if isfile(COOKIE):
        cookie_handler.load(COOKIE)
    return cookie_handler

def get_cookie_processor(cookie_handler):
    """
        Ajoute la gestion automatique de cookie dans urllib2
    """
    return urllib2.HTTPCookieProcessor(cookie_handler)

def get_proxy_handler():
    """
        Désactive la détection automatique du proxy
        en forçant un proxy vide
    """
    return urllib2.ProxyHandler({})

def no_unicode(datas):
    if isinstance(datas, unicode):
        datas = datas.encode('utf-8')
    return datas

def no_unicode_in_dict(datas):
    if isinstance(datas, dict):
        datas = dict([(key, no_unicode(value))for key, value in datas.items()])
    return datas

class PoshRequest:
    """
        Envoie des requêtes à l'API Posh
        :use_cookie: True /False Utilisation de cookie lors des requêtes,
                     mais apparemment on en a pas besoin
    """
    headers = {'user_id':'envoleadmin'}
    use_cookie = False

    def __init__(self, url_script, datas):
        log.debug(" * Requête vers : %s" % (url_script,))
        self.url_script = url_script
#        print datas
        datas = no_unicode_in_dict(datas)
        self.post_datas = urlencode(datas)
        self.opener = None
        self.cookie_handler = None
        self.make_opener()
        self._request = self.build_request()

    def make_opener(self):
        """
            Crée un gestionnaire d'ouverture de requête (cookie, proxy ...)
        """
        handlers = []
        handlers.append(get_proxy_handler())
        if self.use_cookie:
            self.cookie_handler = get_cookie_handler()
            handlers.append(get_cookie_processor(self.cookie_handler))
        self.opener = urllib2.build_opener(*handlers)

    def build_request(self):
        """
            Construit l'objet request
        """
        return urllib2.Request(self.url_script, self.post_datas, self.headers)

    def save_cookies(self):
        """
            sauvegarde les cookies de session
        """
        if self.use_cookie:
            self.cookie_handler.save(COOKIE)

    def request(self):
        """
            éxécute la requête
        """
        try:
            urllib2.install_opener(self.opener)
            handle =  urllib2.urlopen(self._request)
        except IOError, err:
            log.error(" # Erreur : une erreur est survenue lors de la \
requête vers : %s" % (self.url_script,))
            if hasattr(err, 'reason'):
                log.error(' - Echec à la connexion au serveur.')
                log.error(' - Raison: %s' % (err.reason,))
            elif hasattr(err, 'code'):
                log.error(" - Le serveur n'a pu satisfaire la demande.")
                log.error(" - Code d' erreur : %s" % (err.code,))
            raise RequestError, "Une erreur est survenue lors de la requête."
        self.save_cookies()
        return handle.read()
