#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# thumb_api.py
#
# Gestion des onglets
###########################################################################
"""
    Création/modification/suppression des onglets dans Posh
"""
from os.path import join

from posh.logger.logger import getLogger
from posh.config.api import POSH_API_URL, ACTION_DICT

from xml_parse import XmlParse
from request import PoshRequest
from group_api import PoshGroupApi
from posh.error import MissingGroupId, XmlKeyMissing

log = getLogger('api.thumb_api')

def stock_logout_url(logout_url):
    """
        Ajoute une url de déconnexion pour la déconnexion centralisée
    """
    pass

class PoshThumbApi(PoshGroupApi):
    """
        Gestionnaire d'onglet du portail Posh
        Algo d'ajout de page:
            1 - création de l'entrée dans la table pages :
            2 - application aux utiliateurs du groupe (écriture dans profile)
            3 - Si le type est 1 : crée les widgets et les lie au page
    """
    def add_webpage(self, groupname, name, url, description=None, \
                    logout_url='', removable=False, mode="1", imposer='2'):
        """
            Ajoue une page avec url pour le groupe 'groupe'
            :groupname: nom du groupe
            :name: nom de l'onglet
            :description: Description
            :url:url de l'onglet
            :logout_url: Url de logout de l'onglet (prévue pour permettre \
                                                        un logout général)
            :removable: page supprimable ou pas True / False
            :mode: 1 - anonyme / 2 - connecté / 3 - les deux
            :imposer: 1 - seulement pour les nouveaux
                      2 - pour tous les utilisateurs
        """
        log.debug("# Ajout de l'onglet d'url : \
%s pour le groupe %s #" % (url, groupname))
        groupid = self.get_groupid(groupname)
        if description == None:
            description = name
        stock_logout_url(logout_url)

        script_url = join(POSH_API_URL, ACTION_DICT['add_page'])
        if removable:
            removable = '1'
        else:
            removable = '0'
        post_args = dict(
                         group=groupid,
                         name=name,
                         desc=description,
                         mode=mode,
                         param=url,
                         removable=removable,
                         #type=2 mais mot-clé python
                         formaction=imposer,
                         adminkey=self.key)
        post_args['type'] = '2'
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        if xml.success():
            log.debug(" + L'onglet %s a été rajouté dans la base." % (
                                                               name,))
            return True
        else:
            log.error(" - Une erreur est survenue lors de l'insertion de \
l'onglet %s dans la base." %(name,))
            return False

    def add_widgetpage(self):
        """
        $DB->quote($_POST['name']),
        $DB->escape($_POST['nbcol']),
        $DB->escape($_POST['style']),
        $DB->quote($_POST['controls']),
        $DB->escape($_POST['showtype']),
        $DB->quote($_POST['modulealign']),
        $DB->escape($_POST["type"]),
        $DB->quote($_POST["param"]),
        $DB->escape($_POST['removable'])
        """
        pass

    def get_pagesinfo(self, pagename, groupid):
        """
            Renvoie l'id des pages pagename du groupe groupname
            :pagename: nom de l'onglet
            :groupid: id du groupe
        """
        script_url = join(POSH_API_URL, ACTION_DICT['get_pages_info'])
        post_args = dict(
                        pagename=pagename,
                        groupid=groupid,
                        adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.pagesinfo()

    def del_webpage(self, groupname, pagename):
        """
            Supprime les pages pagename du groupe groupname
        """
        try:
            groupid = self.get_groupid(groupname)
        except MissingGroupId:
            log.error(" - Tentative de suppression de page d'un groupe \
inexistant: %s." % (groupname, ))
            groupid = None
        success = True
        if groupid != None:
            try:
                pagesinfo = self.get_pagesinfo(pagename, groupid)
                for page in pagesinfo:
                    if self.del_page(page['pageid'], groupid):
                        success = False
                        log.debug(" + La page %s (%s) du groupe %s a bien \
été supprimée." % (pagename, page['pageid'], groupname))
                    else:
                        log.error(" - Une erreur est survenue à la \
suppression de la page %s (%s) du groupe %s." % (pagename,
                                                 page['pageid'],
                                                 groupname))
            except XmlKeyMissing:
                log.debug(" - Le groupe %s n'a pas de page %s." % (pagename,
                                                            groupname))
        else:
            success = False
        return success

    def del_page(self, pageid, groupid):
        """
            Supprime la page d'id pageid du groupe groupid
        """
        script_url = join(POSH_API_URL, ACTION_DICT['del_page'])
        post_args = dict(
                group=groupid,
                adminkey=self.key)
        post_args['id'] = pageid
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.success()
