#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# usergroup_api.py
#
# Gestion des groupes carnets
###########################################################################
"""
    Création / Test d'existence pour les groupes 'Carnet'
    >>> from posh.api import usergroup_api
    >>> gest = usergroup_api.PoshUserGroupApi()
    >>> iddugroupe = >>> gest.add_usergroup('nomdugroupe', 'nomdelutilisateur')
"""
from os.path import join

from posh.logger.logger import getLogger
from posh.config.api import POSH_API_URL, ACTION_DICT
from posh.error import MissingUser, MissingGroup

from base_api import PoshBaseApi
from user_api import PoshUserApi
from request import PoshRequest
from xml_parse import XmlParse

log = getLogger('api.usergroup_api')

class PoshUserGroupApi(PoshBaseApi):
    """
        Gestionnaire de groupe utilisateur (Carnet)
    """
    def add_usergroup(self, groupname, username):
        """
            :groupname: nom du groupe
            :username: nom de l'utilisateur créateur du groupe
        """
        log.debug("# Ajout du groupe utilisateur %s #" % (groupname,))
        usergroupid = self.get_usergroupid(groupname)
        if usergroupid != '0':
            log.debug("# Le groupe existe déjà sous l'id : %s #" % (
                                                          usergroupid,))
            return usergroupid
        else:
            user_api = PoshUserApi()
            script_url = join(POSH_API_URL, ACTION_DICT['add_usergroup'])
            post_args = dict(act='add',
                             allowunsubscribe='0',
                             pv='1',
                             desc=groupname,
                             name=groupname,
                             user_id=user_api.get_userid(username),
                             adminkey=self.key)
            xml = XmlParse(PoshRequest(script_url, post_args).request())
            usergroupid = xml.usergroupid()
            log.debug(" - L'id du groupe %s dans la base posh \
est : %s" % (xml.usergroupid(), usergroupid))
            return usergroupid

    def get_usergroupid(self, groupname):
        """
            Teste l'existence d'un notebook groupe et renvoie
            l'id si il existe
        """
        script_url = join(POSH_API_URL, ACTION_DICT['usergroup_exist'])
        script_url += "?name=%s" % groupname
        post_args = dict(name=groupname,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.isusergroup()

    def subscribe_usergroup(self, groupname, username):
        """
            Inscrit l'utilisateur username au groupe groupname
        """
        log.debug("# Inscription de %s dans le groupe %s #" % (groupname,
                                                               username))
        # récupération de l'id du groupe
        groupid = self.get_usergroupid(groupname)
        if groupid == '0':
            log.debug(" - Le groupe %s n'existe pas (inscription \
impossible)" % (groupname,))
            raise MissingGroup("Le groupe %s n'existe pas dans Envole." % (
                                                                groupname,))
        # récupération de l'id du user
        user_api = PoshUserApi()
        userid = user_api.get_userid(username)
        if not userid:
            raise MissingUser("Le user %s n'existe pas dans Envole." % (
                                                                username,))
        return self._subscribe_usergroup(groupid, userid)

    def _subscribe_usergroup(self, groupid, userid):
        """
            Inscrit un user dans un groupe à partir de leur id
        """
        script_url = join(POSH_API_URL, ACTION_DICT['add_usergroup'])
        post_args = dict(act='selectnjoin',
                         id=groupid,
                         name=groupid, # attribut exigé (sert à rien)
                         user_id=userid,
                         adminkey=self.key)
        log.debug(post_args)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.usergroupid()

    def del_usergroup(self, groupname, username):
        """
            Supprime un groupe
            Problème : ne supprime que l'inscription du créateur
                       (le groupe reste et les autres sont toujours inscrits)
        """
        # récupération de l'id du groupe
        groupid = self.get_usergroupid(groupname)
        if groupid == '0':
            log.debug(" - Le groupe %s n'existe pas (inscription \
impossible)" % (groupname,))
            raise MissingGroup("Le groupe %s n'existe pas dans Envole." % (
                                                                groupname,))
        # récupération de l'id du user
        user_api = PoshUserApi()
        userid = user_api.get_userid(username)
        if not userid:
            raise MissingUser("Le user %s n'existe pas dans Envole." % (
                                                                username,))
        return self._del_usergroup(groupid, userid)

    def _del_usergroup(self, groupid, userid):
        """
            Supprime un groupe depuis l'id du groupe
            et celui de son créateur (0 si depuis synchro)
        """
        script_url = join(POSH_API_URL, ACTION_DICT['add_usergroup'])
        post_args = dict(act='del',
                         name=groupid,
                         id=groupid,
                         user_id=userid,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.usergroupid()
