#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# xml_parse.py
#
# Outil de parsing xml
#
###########################################################################
"""
    Parsing de xml
"""
from xml.etree import ElementTree

from posh.logger.logger import getLogger
from posh.error import (MissingGroupId, MissingUserId, XmlKeyMissing,
                        UnknownError)

log = getLogger('api.xml_parse')

class XmlParse:
    """
        Outil de parsing xml
    """
    def __init__(self, returned_xml):
        log.debug(" * Parsing de : %s" % (returned_xml,))
        self.parser = ElementTree.XML(returned_xml)

    def success(self):
        """
            renvoie True ou False selon le retour xml
        """
        try:
            return self._key('status') == '1'
        except XmlKeyMissing:
            return False
        except:
            raise UnknownError, "Une erreur inconnue s'est produite à la \
récupération de la balise status dans le xml fournit."

    def groupid(self):
        """
            Renvoie le groupid renvoyé par un xml
        """
        try:
            return self._key('groupid')
        except XmlKeyMissing:
            raise MissingGroupId, "No groupid tag retrieved"
        except:
            raise UnknownError, "Une erreur inconnue s'est produite à la \
récupération du groupid dans le xml fournit."

    def usergroupid(self):
        """
            Renvoie l'id du usergroup retrouvé dans le xml de création
        """
        try:
            return self._key('ret')
        except XmlKeyMissing:
            raise MissingGroupId, "No user_group id tag (<ret></ret>) retrieved"
        except:
                raise UnknownError, "Une erreur inconnue s'est produite à la \
récupération de l'id du usergroup dans le xml fournit."

    def isusergroup(self):
        """
            Renvoie l'id du usergroup si il existe et 0 sinon
        """
        try:
            return self._key('id')
        except MissingGroupId:
            return '0'
        except:
            try:
                return self._key('exist')
            except:
                raise UnknownError, "Une erreur inconnue s'est produite \
au test de l'existence du usergroup dans le xml fournit."


    def userid(self):
        """
            Renvoie le userid renvoyé par un xml
        """
        try:
            return self._key('userid')
        except XmlKeyMissing:
            raise MissingUserId, "No userid tag retrieved"
        except:
                raise UnknownError, "Une erreur inconnue s'est produite à la \
récupération du userid dans le xml fournit."

    def userinfo(self):
        """
            Renvoie les infos utilisateurs
        """
        longname = self._key('long_name')
        userid = self._key('userid')
        typ = self._key('typ')
        lang = self._key('lang')
        return dict(long_name=longname,
                    user=userid,
                    usertype=typ,
                    lang=lang)

    def _pageinfo(self, page):
        """
            Renvoie les infos pour une page depuis un xml
            <id></id>
            <param></param>
            <groupid></groupid>
            <typ></typ>
        """
        pageid = self._key('id', page)
        url = self._key('param', page)
        typ = self._key('type', page)
        return dict(pageid=pageid,
                    url=url,
                    typ=typ)

    def pagesinfo(self):
        """
            Renvoie les infos pour des pages
            <pages><page>
                    </page>
                    <page>...
            </pages>
        """
        pages = []
        for page in self._tags('page'):
            pages.append(self._pageinfo(page))
        return pages

    def _key(self, keyname, etree_tag=None):
        """
            Renvoie la valeur de la balise _key
        """
        if etree_tag == None:
            etree_tag = self.parser
        key_tag = etree_tag.findall(keyname)
        if hasattr(key_tag, '__iter__') and len(key_tag) == 1:
            value = key_tag[0].text
            if value == None:
                value = ''
            return value.strip()
        else:
            raise XmlKeyMissing("La clé %s n'a pas pu être retrouvée \
dans le xml fournit." % (keyname,))

    def _tags(self, keyname):
        """
            Renvoie la liste des balise _keyname sous forme d'objet etree
        """
        key_tags = self.parser.findall(keyname)
        if hasattr(key_tags, '__iter__') and len(key_tags) >= 1:
            return key_tags
        else:
            raise XmlKeyMissing(keyname)
