#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    Connecteur de base pour l'accès aux base de données
"""
from _mysql_exceptions import Warning as sqlWarning
import warnings
import MySQLdb
from MySQLdb.constants import FIELD_TYPE, FLAG
from MySQLdb.converters import conversions


from posh.logger.logger import getLogger
log = getLogger('client.posh.db.basedb')

warnings.simplefilter('ignore', sqlWarning)

# On veut pas nombre, on veut que des strings !!!!
CONV_DICT = conversions.copy()
for key, value in CONV_DICT.items():
    if value == int or value == long or value == float:
        CONV_DICT[key] = str

def stringify(element):
    """
        element : données à formatter pour transformer les int en string
        TODO : devrait faire pareil pour les autres types de données
    """
    if isinstance(element, int):
        return str(element)
    elif isinstance(element, tuple) or isinstance(element, list):
        element = [stringify(elem) for elem in element]
        return element
    else:
        return element

class DB(object):
    """
        Objet de base pour la connexion aux base de données
        connect_db : ouvre une connexion à la base et initialise un curseur
        close_db : ferme la connexion à la base

    """
    CREATE_DB = """CREATE DATABASE IF NOT EXISTS `%s` DEFAULT \
CHARACTER SET utf8;"""
    ADD_TABLE = """CREATE TABLE IF NOT EXISTS '%s'(%s)"""

    def __init__(self, dbinfos, dbname='poshprofile'):
        if '' in dbinfos.values():
            raise Exception, "Missing database infos values"
        self.db = None
        self.dbinfos = dbinfos
        self.init_db(dbname)

    def init_db(self, dbname='poshprofile'):
        """
        crée la base de données si elle n'existe pas
        """
        self.connect_db(True)
        self.db.query(self.CREATE_DB % (dbname))
        self.close_db()

    def connect_db(self, mysql=False):
        """
            ouvre une connexion vers la base de données
        """
        if mysql:
            dbinfos = self.dbinfos.copy()
            dbinfos.pop('db')
            self.db = MySQLdb.connect(conv=CONV_DICT, **dbinfos)
        else:
            self.db = MySQLdb.connect(conv=CONV_DICT, **self.dbinfos)

    def close_db(self):
        """
            Ferme la connexion
        """
        self.db.close()

    def add_table(self, add_table_query):
        """
            ajoute une table dans la base si elle n'existe pas encore
        """
        self.connect_db()
        cursor = self.db.cursor()
        cursor.execute(add_table_query)
        self.close_db()

    def query(self, query_str):
        """
            exécute une requête dans la base
        """
        self.connect_db()
        cursor = self.db.cursor()
        cursor.execute(query_str)
        result = list(cursor.fetchall())
        return stringify(result)

    def insert(self, query_str):
        """
            exécute une requête d'insert dans la base
        """
        self.connect_db()
        cursor = self.db.cursor()
        try:
            cursor.execute(query_str)
            self.db.commit()
        except:
            self.db.rollback()
        self.close_db()
        return True

    def delete(self, query_str):
        """
            éxécute une requête de suppression
        """
        return self.insert(query_str)

    def _insert(self, template, query_list, connect=True):
        """
            insert un élément dans la base selon le template
        """
        if not isinstance(query_list, tuple):
            query_list = tuple(query_list)
        log.debug("Mysql query : * template : %s" % (template,))
        log.debug("              * arguments : %s" % str(query_list))
        if connect:
            self.connect_db()
        cursor = self.db.cursor()
        try:
            cursor.execute(template, query_list)
            self.db.commit()
        except:
            self.db.rollback()
        if connect:
            self.close_db()

    def _update(self, template, query_list, connect=True):
        """
            update un élément de la base selon le template
        """
        if not isinstance(query_list, tuple):
            query_list = tuple(query_list)
        log.debug("Mysql query : * template : %s" % (template,))
        log.debug("              * arguments : %s" % str(query_list))
        if connect:
            self.connect_db()
        cursor = self.db.cursor()
        try:
            cursor.execute(template, query_list)
            self.db.commit()
        except:
            self.db.rollback()
        if connect:
            self.close_db()

    def _query(self, template, query_list, connect=True):
        """
            éxécute une requête et renvoie le résultat
            connect : connection à la base avant requeête
        """
        if isinstance(query_list, str):
            query_list = (query_list,)
        elif not isinstance(query_list, tuple):
            query_list = tuple(query_list)
        log.debug("Mysql query : * template : %s" % (template,))
        log.debug("              * arguments : %s" % str(query_list))
        if connect:
            self.connect_db()
        cursor = self.db.cursor()
        cursor.execute(template, query_list)
        result = cursor.fetchall()
        if connect:
            self.close_db()
        return result
