#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Classe de base pour la connexion à la base de profil
#
###########################################################################
"""
    Classe de base pour la connexion à une base mysql
    TableDB : Classe générique pour
              l'insertion/modification/suppression
              dans une base
    LinkDB : Classe générique pour l'ajout de lien dans une table de jointure
"""
import MySQLdb
from copy import deepcopy

from posh.db.basedb import DB

class TableDB(DB):
    """
    Modèle de gestionnaire de table sql
    """
    table_name = ""
    CREATE_TABLE = ""
    FIELDS = []
    ADDFIELD_QUERY = ""
    FIELD_QUERY = """SELECT * FROM `%s` %%s """ % table_name
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """ % table_name
    ADDFORMATTEDFIELD_QUERY = ""


    def __init__(self, dbinfos):
        """
        ajoute la table si elle existe pas encore
        """
        super(TableDB, self).__init__(dbinfos)
        self.add_table(self.CREATE_TABLE)

    def _add_field(self, field_list):
        """
            ajoute un champ dans la table
            field_list : liste des valeurs de champ
        """
        # escape les caractères
        field_list = [MySQLdb.escape_string(str(field)) for field in field_list]
        self.insert(self.ADDFIELD_QUERY % ','.join(
                              map(lambda x: '"%s"' % (x,), field_list) ))

    def _add_formated_field(self, field_dict):
        """
            ajoute un champ dans la table
            field_list : liste des valeurs de champ
        """
        # escape les caractères
        field_dict = dict([(key, MySQLdb.escape_string(str(value)))
                                     for key, value in field_dict.items()])
        self.insert(self.ADDFORMATTEDFIELD_QUERY % field_dict)

    def _get_fields(self, order=None, **dbkeys):
        """ renvoie les entrée de table sql correspondant
            aux clés-valeurs indiquées dans dbkeys
            triés selon le critère order
        """
        # on filtre que les champs existent et ne sont pas vides
        for field, value in dbkeys.items():
            if field not in self.FIELDS or not value:
                dbkeys.pop(field)
        if order not in self.FIELDS and order:
            raise Exception, "Clé inexistante : %s" % order

        # construction de la requete
        query_opts = ""
        if dbkeys:
            query_opts = 'WHERE '
            query_opts += ' and '.join(["%s='%s'" % (
                            key, MySQLdb.escape_string(str(value)))
                                           for key, value in dbkeys.items()])
        if order:
            query_opts += " ORDER BY %s" % order
        query_str = self.FIELD_QUERY % query_opts
        return self.query(query_str)

    def format_field(self, field_db):
        """
        formatte la sortie bdd en format utilisable par le mapper
        """
        field = dict([(self.FIELDS[indice], key)
                            for indice, key in enumerate(field_db)])
        return field

    def _get_formated_fields(self, order=None, **dbkeys):
        """
        renvoie les champs de la base formatté en dico {clé:valeur}
        """
        return [self.format_field(field)
                            for field in self._get_fields(order, **dbkeys)]

    def _del_field(self, **dbkeys):
        """
        supprime l'entrée dans la base du thumbs thumb_name avec l'url url
        """
        # on filtre que les champs existent et ne sont pas vides
        for field, value in dbkeys.items():
            if field not in self.FIELDS or not value:
                dbkeys.pop(field)
        if dbkeys:
            query_opts = ' and '.join(["%s='%s'" % (
                            field, MySQLdb.escape_string(str(value)))
                                       for field, value in dbkeys.items()])
            query_str = self.DELETE_QUERY % query_opts
            self.delete(query_str)
            return True
        raise Exception, "Error: missing parameters for deleting \
entry in database"

    def purge_table(self):
        """
        supprime tous les éléments de la table thumbs
        """
        query_str = """DELETE FROM %s""" % self.table_name
        self.delete(query_str)
        return True

    def get_keys(self):
        """ renvoie les clés de la table sans l'id """
        return deepcopy(self.FIELDS[1:])

class LinkDB(TableDB):
    """
        Modèle de table de link
    """
    table_name = ''
    FIELDS = ["id1", "id2"]
    ADDFIELD_QUERY = """INSERT into `%s` (%s) VALUES (%%s)""" % (table_name,
                                                            ','.join(FIELDS))
    CREATE_TABLE = """CREATE TABLE IF NOT EXISTS `%s` (
                        `id1` MEDIUMINT NOT NULL COMMENT 'identificateur 1',
                        `id2` MEDIUMINT NOT NULL COMMENT 'identificateur 2'
                      )ENGINE = MYISAM DEFAULT CHARSET=utf8 \
COLLATE=utf8_unicode_ci COMMENT='table modele de lien';""" % table_name
    FIELD_QUERY = """SELECT * FROM `%s` %%s """ % table_name
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """ % (table_name)

    def add_link(self, id1, id2=None):
        """ rajoute un lien entre l'élément id1 et l'élément id2
        """
        link = [id1, id2]
        self._add_field(link)
        return True

    def get_links(self, **attrs):
        """
            liste les liens liés à id1 ou id2 ou les 2
            et les renvoie sous une forme [{'id1':valeur, 'id2':valeur}]
        """
        if not set(attrs.keys()).issubset(set(self.FIELDS)):
            raise Exception, "Erreur : requête vers des clés non existante."
        return [self.format_field(field_db)
                                  for field_db in self._get_fields(**attrs)]

    def del_link(self, **attrs):
        """
            supprime un lien entre id1 et id2
        """
        if not set(attrs.keys()).issubset(set(self.FIELDS)):
            raise Exception, "Erreur : requête vers des clés non existante."
        return self._del_field(**attrs)

