#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    Classe générique pour la connexion à une table mysql
    Depuis la configuration de la table,
    génère les requêtes sql nécessaires
"""
import MySQLdb
from copy import deepcopy

from posh.db.baseprofile import TableDB

class TableDBConstructor(TableDB):
    """
        constructeur d'interface de base de données
        mytable: {'name':nomdelatable
           'fields': ({
              '_type':type de champ,
              'libelle':'libellé du champ',
              'name':nom du champ,
              'default_value': valeur par défaut du champ, (facultatif
              'unique': 'ok' (facultatif
              },
            )
           'libelle':Intitulé de la table
            }
    """
    ADD_FIELD_QUERY = """INSERT into `%s` (%s) VALUES (NULL, %%s)"""
    ADDFORMATTEDFIELD_QUERY = """INSERT into `%s` (%s) VALUES (NULL, %s)"""
    CREATE_TABLE = """CREATE TABLE IF NOT EXISTS `%s` (
                        %%s
    )ENGINE = MYISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci \
COMMENT='%s posh';"""
    UPDATE_QUERY = """UPDATE `%s` SET %%s WHERE %%s='%%s'"""
    FIELD_QUERY = """SELECT * FROM `%s` %%s """
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """


    def __init__(self, dbinfos, mytable):
        self.FIELDS = [field['name'] for field in mytable['fields']]
        self.modifiable = [field['name'] for field in mytable['fields']
                                            if field.has_key('modify')]

        self.fields_datas = mytable['fields']
        self.table_name = mytable['name']
        self.table_libelle = mytable['libelle']
        self.ADDFIELD_QUERY = self.ADD_FIELD_QUERY % (self.table_name,
                                                      ','.join(self.FIELDS))
        # construction de string "%(nomduchamp)s"
        # pour les insertions dans la base
        temporary = "," . join(["'%(" + "%s" % key +")s'"
                              for key in self.FIELDS[1:]])

        self.ADDFORMATTEDFIELD_QUERY = self.ADDFORMATTEDFIELD_QUERY % (
                                                        self.table_name,
                                                        ','.join(self.FIELDS),
                                                        temporary)

        self.CREATE_TABLE = self.CREATE_TABLE % (self.table_name,
                                                 self.table_libelle)
        self.make_create_table_query(mytable['fields'])
        self.FIELD_QUERY = self.FIELD_QUERY % self.table_name
        self.DELETE_QUERY = self.DELETE_QUERY % self.table_name
        self.UPDATE_QUERY = self.UPDATE_QUERY % self.table_name

        # instanciation des méthodes nommées
        setattr(self, "get_%s" % self.table_name, self._get_formated_fields)
        setattr(self, "del_%s" % self.table_name, self._del_field)
        setattr(self, "add_%s" % self.table_name, self._add_formated_field)

        super(TableDBConstructor, self).__init__(dbinfos)

    def make_create_table_query(self, fields):
        """
            construit les requêtes d'initialisation de table
            pour chaque champ, gestion des types, de l'unicité
            et des valeurs par défaut
        """
        query = []
        for field in fields:
            if field['_type'] == 'autoincrement':
                field_query = "`%s` MEDIUMINT NOT NULL AUTO_INCREMENT \
PRIMARY KEY" % (field['name'])
            else:
                if 'INT' in field['_type']:
                    field_query = "`%(name)s` %(_type)s NOT NULL %%s" % field
                elif 'CHAR' in field['_type']:
                    field_query = "`%(name)s` %(_type)s CHARACTER SET utf8 \
COLLATE utf8_bin NOT NULL %%s" % field
                if field.has_key('DEFAULT'):
                    default = "DEFAULT %s" % str(field['DEFAULT'])
                    field_query = field_query % default
                else:
                    field_query = field_query % ""
            query.append(field_query)
        uniques = []
        for field in fields:
            if field.has_key('UNIQUE'):
                uniques.append("`%s`" % field['name'])
        if uniques:
            unique_query = "UNIQUE(%s )" % ',\n'.join(uniques)
            query.append(unique_query)
        self.CREATE_TABLE = self.CREATE_TABLE % ',\n'.join(query)

    def get_fields(self):
        """ renvoie les champs et leur libellés
            sauf le id
        """
        return deepcopy([field for field in self.fields_datas
                                   if field['_type'] != 'autoincrement'])

    def get_id_key(self):
        """
            renvoie le nom de la clé qui sert d'identification
        """
        for field in self.fields_datas:
            if field['_type'] == 'autoincrement':
                return field['name']
        else:
            return "id"

    def modify(self, _id, **dbkeys):
        """
            modifie une valeur pour l'entrée de la table d'id _id
        """
        idkey = self.get_id_key()
        args = ''
        for key, value in dbkeys.items():
            if key not in self.modifiable:
                raise Exception, "Erreur de mofication d'entrée :  Cette clé \
                                                       n'est pas référencée"
            if args:
                args += ","
            args += "%s='%s'" % (key, MySQLdb.escape_string(str(value)))
        update_query = self.UPDATE_QUERY % (args, idkey, _id)
        self.insert(update_query)
