# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# exceptions spécifiques
#
###########################################################################
"""
exceptions particulières
"""

class FileNotFoundError(Exception):
    pass

class UnknownProfile(Exception):
    """Exception levée quand on rencontre un acteur (rôle ou user) incconu"""
    def __init__(self, profile):
        Exception.__init__(self)
        self.profile = profile

    def __str__(self):
        return "No such actor: %r" % self.actor

class UnknownThumb(Exception):
    """Exception levée quand on rencontre un onglet incconu"""
    def __init__(self, thumbs):
        Exception.__init__(self)
        self.thumbs = thumbs

    def __str__(self):
        return "No such thumbs: %r" % self.thumbs

###################
# Erreur pour l'API

class UnknownError(Exception):
    """
        Erreur inconnue
    """
    pass
class UnknownAction(Exception):
    """
        Soulevé lors de l'appel à une action inconnue
    """
    pass
class RequestError(Exception):
    """
        Erreur lors d'une requête http
    """
    pass
class MissingFile(Exception):
    """
        Erreur lors de l'absence d'un fichier
    """
    pass
class XmlException(Exception):
    """
        Erreur de parsing xml
    """
    pass
class XmlKeyMissing(XmlException):
    """
        Erreur soulevée en l'absence d'une clé
    """
    pass
class MissingGroupId(XmlException):
    """
        Erreur soulevée lorsqu'il manque l'attribut group_id
        dans le xml renvoyé
    """
    pass
class MissingUserId(XmlException):
    """
        Erreur soulevée lorsqu'il manque l'attribut user_id
        dans le xml renvoyé
    """
    pass
class MissingUser(Exception):
    """
        Soulevée lorsqu'un utilisateur n'existe pas dans la base d'envole
    """
    pass
class MissingGroup(Exception):
    """
        Soulevée lorsqu'un groupe n'existe pas dans la base d'envole
    """
    pass

################################
# Erreur d'accès base de données
class AddDBError(Exception):
    """
        Erreur d'insertion , element : élément dont l'insertion a échouée
    """
    def __init__(self, table, element):
        self.element = element
        self.table = table

class DelDBError(AddDBError):
    """
        Erreur de suppression d'élément dans une table
    """
    def __init__(self, table, elementid):
        self.table = table
        self.elementid = elementid

class GetDBError(Exception):
    """
        Erreur de récupération d'élément dans une table de jointure
    """
    def __init__(self, table, element):
        self.table = table
        self.element = element

class AddLinkDBError(Exception):
    """
        Erreur d'ajout de lien dans une table de jointure
    """
    def __init__(self, linktablename, table1, table2, key1, key2, link):
        self.linktable = linktablename
        self.element1 = {'table':table1, 'key':key1, 'id':link[key1]}
        self.element2 = {'table':table2, 'key':key2, 'id':link[key2]}

class DelLinkDBError(AddLinkDBError):
    """
        Erreur de suppression de lien
    """
    pass

class GetLinkDBError(GetDBError):
    """
        Erreur de récupération de lien dans une table de jointure
    """
    def __init__(self, linktablename, table1, table2, key1, key2, link):
        self.linktable = linktablename
        self.element = {'table':table1, 'key':key1, 'id':link[key1]}

class ModDBError(AddDBError):
    """
        Erreur de modification d'entrée dans la base
    """
    def __init__(self, table, elem_id, modkeys):
        self.table = table
        self.element_id = elem_id
        self.modkeys = modkeys
