#-*-coding:utf-8-*-
"""
    Outil d'exportation d'onglet:
    Exporte un onglet vers la base profil en appelant le serveur
"""
from logging import getLogger

from posh.db.basedb import DB
from posh.config.config import posh_db_config
#import permettant de s'assurer de l'initialisation des loggers posh
from posh.logger import logger

log = getLogger('client.posh.thumbs.exportthumbs')

TABQUERY = """SELECT name,seq,type,param,id FROM profile WHERE \
user_id=%s AND id=%s """

class ExportThumbs(DB):
    """ exportateur d'onglet """

    def __init__(self, username, session_pt, user_id, thumb_id, thumb_name,
                                         thumb_indice, xmlrpc_server, myip):
        """
            user_id : id de l'utilisateur admin dans la base
            thumb_id : id de l'onglet dans la base
            session_pt : pt du serveur cas pour le serveur de profil
            thumb_name : nom de l'onglet pour dans la base
            thumb_indice : son indice utilisé pour l'ordre des onglets
                           dans la page du user
            xmlrpc_server : objet xmlrpclib.Server
            myip : ip du client de profil
        """
        log.debug("*** Initialisation de la classe d'exportation d'onglet ***")
        log.debug("Nom de l'utilisateur appelant : %s" % (username,))
        log.debug("Ticket de session de l'utilisateur : %s" % (session_pt,))
        log.debug("Id de l'utilisateur dans la table 'posh' : %s" % (user_id,))
        log.debug("Id de l'onglet à exporter : %s" % (thumb_id,))
        log.debug("Nom de l'onglet : %s" % (thumb_name,))

        super(ExportThumbs, self).__init__(posh_db_config, dbname='posh')
        self.username = username
        self.session_pt = session_pt
        self.user_id = user_id
        self.thumb_id = thumb_id
        self.thumb_name = thumb_name
        self.thumb_indice = thumb_indice
        self.server = xmlrpc_server
        self.myip = myip

    def _get_thumb(self):
        """
            renvoie les données liées à l'onglet à exporter
        """
        log.debug(" + Récupération des informations de l'onglet \
dans la base de posh")

        try:
            tab = self._query(TABQUERY, (self.user_id, self.thumb_id))
            result = list(tab[0])
        except Exception:
            result = []
            log.exception("Problème à la récupération de la description \
de l'onglet dans la base de posh.")

        # si l'indice doit être modifié
        try:
            if int(self.thumb_indice) >= 0:
                result[1] = self.thumb_indice
        except Exception:
            log.exception("On essaye d'insérer un onglet dont \
l'indice n'est pas un nombre.")

        # on insère le nom pour dans la table de profil
        result.insert(0, self.thumb_name)
        log.debug(" + Description de l'onglet %s" % str(result))
        return result

    def export_thumb(self):
        """
            exporte un onglet
        """
        thumb = self._get_thumb()
        try:
            return_str = self.server.export_thumb(self.username,
                                                  self.session_pt,
                                                  self.myip,
                                                  self.thumb_name,
                                                  thumb)

            #return_str = return_str.encode('utf-8')
            log.debug(" + Le serveur de profil a renvoyé la réponse \
suivante : %s" % return_str)
        except Exception:
            log.exception("Echec de l'export d'onglet \
vers la base de profil")
            return_str = "<err>Erreur inconnue à l'export de l'onglet</err>"
        return return_str
