#-*-coding:utf-8-*-
"""
    outil de gestion des onglets pour posh

    récupère les onglets auprès du gestionnaire de profil
    les compare aux onglets métiers déjà écrits dans la base
    pour cet utilisateur et rajoute ceux qui manquent

    gère également les modules
"""
from posh.db.basedb import DB
from posh.thumbs.thumb_tools import _next_module_id, _set_all_seqs, \
is_widget_thumb
from posh.perm.rule import Thumb, Module

from posh.config.config import posh_db_config
from posh.config import thumb_sql as sql
# import servant à assurer l'initialisation des loggers
from posh.logger.logger import getLogger

log = getLogger('client.posh.thumb.initthumb')

class InitThumbs(DB):
    """
        gestionnaire de profil
        Récupère les descriptions des onglets en fonction des infos user
        écrit dans la table

        Dans la base:
            seq: ordre des onglets
            id: id de l'onglet (par utilisateur)
        @user_datas : informations de l'utilisateur provenant du serveur SSO
                      voir configuration du serveur SSO
        Un onglet c'est :
            - un user_id
            - une position (ordre des onglets)
            - un libelle
            - une icone         (facultative)

            - un type de page   (avec widget, url)
            - une url
    """
    # requête pour la récupération des onglets:
    def __init__(self, user_datas, username=None, xmlrpc_server=None):
        """
            @user_datas : dictionnaire décrivant les données de l'utilisateur
                          en cours de connexion
        """
        log.debug("************ Initialisation de InitThumbs ************")
        log.debug("Initialisation des onglets pour l'utilisateur : %s" % username)
        log.debug("Données de l'utilisateur : %s" % user_datas)
        super(InitThumbs, self).__init__(posh_db_config, dbname='posh')
        if user_datas.has_key('infos'):
            self.user_datas = user_datas['infos']
        else:
            self.user_datas = user_datas

        if username == None:
            self.username = user_datas['infos']['uid'][0]
        else:
            if isinstance(username, list):
                username = username[0]
            self.username = username
        self._get_user_id()
        self.server = xmlrpc_server

    def _get_user_id(self):
        """
            récupère l'user_id de l'utilisateur
            dans la base à l'aide des données utilisateurs
        """
        uid = self.username
        try:
            self.user_id = self._query(sql.USER_ID_QUERY, (uid,))[0][0]
            log.debug("user_id récupéré pour %s : %s" % (uid, self.user_id))
        except Exception:
            log.exception("- Problème à la récupération de l'id de l'utilisateur")

## onglets internes et externes
    def _get_user_inttabs(self):
        """onglets internes à l'utilisateur
            (ceux résultant du querry de la db posh)
        """
        result = self._query(sql.GETSPECTAB_QUERY, (self.user_id,))
        thumbs = set()
        for row in result:
            thumbs.add(Thumb(
                name=str(row[sql.PROFILE_TABLE_INDICES['eolename']]),
                description=str(row[sql.PROFILE_TABLE_INDICES['name']]),
                seq=str(row[sql.PROFILE_TABLE_INDICES['seq']]),
                _type=str(row[sql.PROFILE_TABLE_INDICES['_type']]),
                param=str(row[sql.PROFILE_TABLE_INDICES['param']])
                    )
                )
        return thumbs

    def _get_user_exttabs(self):
        """
            renvoie les onglets externes de l'utilisateur
            ('onglets métiers')
            en fonction de son profil
            Profile --user_datas--> gestion de profil --profil-->
                            gestion d'onglet --onglet--> Profile
        """
        external_thumbs = [Thumb(*thumb)
                                 for thumb in self.server.get_thumbs(
                                       self.username, self.user_datas)
                          ]
        external_thumbs = _set_all_seqs(external_thumbs)
        return set(external_thumbs)

## insertion des onglets dans la base
    def _get_insert_tabs(self, ext_thumbs):
        """collecte les onglets des utilisateurs (internes et externes)
            c'est le complémentaire d'une différence symétrique

            attention, si 2 onglets interne et externe sont identiques
            niveau objet, il ne le sont pas forcément au niveau de 'tout'
            les attributs de l'objet donc
        ext.symmetric_difference(int) != int_thumbs.symmetric_difference(ext_thumbs)
            l'un renvoie les éléments de ext_thumbs, l'autre ceux de int_thumbs
            l'ext_thumb contient l'attribut admin_profile_id pas un int_thumb

        int = onglets internes
        ext = onglets externes
        sd = ext.symmetric_difference(int)
        ensuite on prend le complémentaire de sd par rapport à int

        """
        int_thumbs = self._get_user_inttabs()
        sd = ext_thumbs.symmetric_difference(int_thumbs)
        return sd.difference(int_thumbs), ext_thumbs.intersection(int_thumbs)

    def _get_thumb_id(self, thumb):
        """ recupère l'id d'un onglet dans la table profile de posh """
        return self._query(sql.GET_THUMBID,
                           (self.user_id, thumb.description))[0][0]

    def insert_tabs_and_modules(self):
        """
            1 - insertion des onglets extérieurs (provenant du gestionnaire
                                                                 de profil)
                les onglets peuvent pointer vers une url externe ou contenir
                                                                 des widgets
            2 - insertion et mise à jour des widgets
        """
        log.debug("appel à insert_tabs_and_modules")
        try:
            # récupération des onglets extérieurs (du gestionnaire de profil)
            ext_thumbs = self._get_user_exttabs()

            # Onglets à insérer
            tabs_to_insert = list(self._get_insert_tabs(ext_thumbs)[0])
            # Onglets déjà dedans
            already_in = self._get_insert_tabs(ext_thumbs)[1]

            log.debug(' + Onglets déjà présent dans la base de données')
            for i in already_in:
                log.debug("   - %s : %s" % (str(i.name),
                                            str(i.admin_profile_id)))
            log.debug(' \n')
            log.debug(' + Onglets à insérer dans la base de données')
            for i in tabs_to_insert:
                log.debug("   - %s : %s"%(str(i.name),
                                          str(i.admin_profile_id)))
            log.debug(' \n')

            # mise à jour des séquences
            for thumb in tabs_to_insert:
                log.debug(" + Insertion de l'onglet : %s" % thumb.name)
                 # ajout de param à l'onglet pour son écriture dans la base
                _thumb = self._update_thumb(thumb.to_dbfield())
                # insertion d'onglet
                try:
                    user_profile_id = self.insert_field(_thumb)
                    log.debug(" + Id de l'onglet dans la base profile \
: %s" % user_profile_id)
                except Exception:
                    log.exception("- Problème lors de l'insertion d'onglet \
dans la base." )
                # insertion des widgets
                try:
                    # user_profile_id est l'id de la page (onglet)
                    # dans la base profile
                    # il est utilisé pour lier la page à ses modules
                    self.set_user_modules(thumb, user_profile_id)
                except Exception:
                    log.exception("- Problème lors de l'écriture des widgets de \
la page %s." % (thumb.name,))

            # insertion/mise à jour  des widgets
            try:
                self.update_user_modules(already_in)
            except Exception:
                log.exception("- Problème lors de la mise à jour des widgets \
de l'utilisateur")
            log.debug("--> Fin de l'insertion des onglets et de leurs modules")
        except Exception:
            log.exception("- Problème à l'insertion d'onglet dans \
insert_tabs_and_modules.")


    def insert_field(self, thumb):
        """ insertion d'un onglet dans le profil de l'utilisateur
        """
        seq = thumb[2]
        result = self._query(sql.SELECT_SEQ_QUERY , (self.user_id, seq))
        self.connect_db()
        # on met à jour les séquences des onglets au milieu
        # desquels on veut intercaler
        log.debug("  + Mise à jour des séquences des onglets déjà présent")
        if str(seq) in [row[sql.PROFILE_TABLE_INDICES['seq']]
                                            for row in result]:
            for row in result:
                new_tab_seq = int(row[sql.PROFILE_TABLE_INDICES['seq']])+1
                uid = row[sql.PROFILE_TABLE_INDICES['user_id']]
                tid = row[sql.PROFILE_TABLE_INDICES['id']]
                name = row[sql.PROFILE_TABLE_INDICES['name']]
                self._update(sql.UPDATETAB_QUERY, (new_tab_seq,
                                                   uid,
                                                   tid,
                                                   name),
                                                   connect=False)
        # on insert
        log.debug("  + Insertion effective de l'onglet dans la base de données")
        self._insert(sql.INSERTTAB_QUERY , thumb, connect=False)
        self.close_db()
        return self._query(sql.GETTABID_QUERY, thumb, connect=True)[0][0]

    ### insertion de modules dans la base
    def set_user_modules(self, thumb, user_profile_id):
        """
            définit les modules de l'utilisateur depuis ceux
            de la page de l'admin pour une page avec widgets
            user_profile_id : id de l'onglet dans la table de l'utilisateur
        """
        log.debug("  + Ajout des widgets dans la page")
        if is_widget_thumb(thumb.admin_profile_id):
            admin_modules = self._get_user_thumbs_modules(
                                        thumb.admin_profile_id)
            self.connect_db()
            for module in admin_modules:
                self._insert_module(module, user_profile_id)
            self.close_db()

    ## update des modules dans la base
    def update_user_modules(self, thumbs):
        """ met à jour les modules des pages du user """
        log.debug("  + Mise à jour des modules (widgets) dans les pages \
de l'utilisateur")
        for thumb in thumbs:
            log.debug("  + Onglets déjà dedans : %s" % (thumb.name))
            # on update les widgets de la page
            if is_widget_thumb(thumb.admin_profile_id):
                log.debug('  + Update des widgets de %s' % thumb.name)
                self._update_user_module(thumb, self._get_thumb_id(thumb))

    def _update_user_module(self, thumb, user_profile_id):
        """ met à jour les modules de l'utilisateur pour une page avec widget
            user_profile_id : id de l'onglet dans la table de l'utilisateur
        """
        admin_modules = self._get_user_thumbs_modules(thumb.admin_profile_id)
        usr_modules = self._get_user_thumbs_modules(user_profile_id,
                                                        admin=False)
        perso_modules = self._get_user_thumbs_modules(user_profile_id,
                                            admin=False, personnal=True)

        self._set_modified_modules(admin_modules, usr_modules,
                                   perso_modules, user_profile_id)

    def _set_modified_modules(self, admin_modules, usr_modules,
                                    perso_modules, profile_id):
        """
            On met à jour les modules de l'utilisateur
        """
        # on transforme les listes de modules en dico avec l'id uniq comme clé
        adm_dict_modules = dict([(mod[10], mod) for mod in admin_modules])
        usr_dict_modules = dict([(mod[10], mod) for mod in usr_modules])
        usr_pers_dict_modules = dict([(mod[10], mod) for mod in perso_modules])
        self.connect_db()
        for key, value in adm_dict_modules.items():
            if key in usr_dict_modules:
                # cas où le module a déjà été écrit dans la table du user
                log.debug("  + Module (widget) déjà présent dans la page \
de l'utilisateur")
                comp = Module(value) == Module(usr_dict_modules[key])
                if not comp:
                    self._update_module(value, profile_id, key)

            elif key in usr_pers_dict_modules:
                # cas où un module perso a déjà le même id
                log.debug("  + Module (widget) à rajouter mais un \
widget personnel a le même id")
                # modification de l'id du module perso pour pouvoir mettre
                # le module imposé à la place
                new_id = _next_module_id(
                                adm_dict_modules.keys(),
                                usr_dict_modules.keys(),
                                usr_pers_dict_modules.keys()
                                )
                self._update_module_id(new_id, profile_id, key)
                # insertion de l'onglet imposé
                self._insert_module(value, profile_id)
            else: # cas où le module imposé va être écrit
                log.debug("  + Module (widget) non présent dans la page \
de l'utilisateur")
                self._insert_module(value, profile_id)
        self.close_db()

    def _update_module(self, module, profile_id, uniq):
        """ met à jour l'entrée dans la base module pour l'élément module
        """
        try:
            # on update les champs item_id et variables
            item_id, variables = module[0], module[9]
            self._update(sql.UPDATE_MODULE, (item_id,
                                             variables,
                                             self.user_id,
                                             profile_id,
                                             uniq),
                                 connect=False)
        except Exception:
            log.exception("- Problème à la mise à jour d'un module (widget)")

    def _update_module_id(self, new_id, profile_id, uniq):
        """ met à jour l'id du module d'un utilisateur """
        try:
            self._update(sql.UPDATE_ID_MODULE, (new_id,
                                                self.user_id,
                                                profile_id,
                                                uniq),
                                    connect=False)
        except Exception:
            log.exception(" - Problème à la mise à jour de l'id du module")

    def _insert_module(self, module, profile_id):
        """ insert un module dans la page d'id profile_id """
        try:
            to_insert = self._make_it_a_user_module(module, profile_id)
            self._insert(sql.INSERTMODULE_QUERY, to_insert, connect=False)
        except Exception:
            log.exception(" - Problème à l'insertion d'un onglet")

    def _make_it_a_user_module(self, listmodule, user_profile_id):
        """ adapte une entrée de la table module à un utilisateur
        """
        listmodule = list(listmodule)
        listmodule[1] = self.user_id
        listmodule[2] = user_profile_id
        # utilisation d'une entrée de table inutilisée
        # pour tagger les modules imposés
        listmodule[8] = 'm'
        return listmodule

    def _get_user_thumbs_modules(self, user_profile_id, admin=True,
                                                    personnal=False):
        """ récupère les modules liés à un onglet d'id user_profile_id,
            dans la base module de posh
            si admin on cherche chez l'admin sinon on utilise l'id de
            l'utilisateur courant pour chercher ses onglets profilés
            si personnal, on prend les onglets perso de l'utilisateur
        """
        if personnal:
            tmpl = sql.USER_PERSO_THUMB_MODULES_QUERY % self.user_id
        elif admin:
            tmpl = sql.ADMIN_THUMB_MODULES_QUERY
        else:
            tmpl = sql.USER_PROFILED_THUMB_MODULES_QUERY % self.user_id
        return self._query(tmpl, (str(user_profile_id),))

    ## méthodes utiles
    def _update_thumb(self, thumb):
        """ ajoute des parametres au thumb
            user_id, le tag pour l'onglet métier,
            les tailles de la matrice de position pour le cas où c'est
            des pages avec widget
        """
        # on sort le admin_profile_id qui ne sert que
        # pour la recuperation d'éventuel widget
        thumb = thumb[:5]
        thumb.append(self.user_id)
        thumb.append('metier')
        thumb.append('3')
        thumb.append('5')
        return thumb
