#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    client xmlrpc renvoyant les description des onglets
    de l'utilisateur posh courant appel avec :
        python client.py <infos utilisateur au format Json>
"""
import json
from posh.xmlrpc.client import xmlrpclib
import sys

from posh.config.config import XMLRPC_IP_SERVER, XMLRPC_PORT, LOCAL_IP
from posh.thumbs.initthumbs import InitThumbs
from posh.thumbs.exportthumbs import ExportThumbs
from posh.logger.logger import getLogger

log = getLogger('client.xmlrpc-posh.client')

def encode_value(result):
    """
        Transforme le résultat en string pour le renvoie à posh
    """
    try:
        return result.encode('utf-8')
    except:
        return result

def return_result(result):
    """
        renvoie le résultat sous forme de print pour passage au php
    """
    result = encode_value(result).strip()
    log.debug(" + On renvoie un résultat au php : ")
    log.debug(result)
    print result

class InitException(Exception):
    """
        Erreur dans l'initialisation du client
    """
    message = "Une erreur s'est produite à l'initialisation du client \
(des informations sont manquantes)."

class EnvoleProfilClient:
    """
        client pour les appels profil utilisateur
        desktop: bureau utilisateur
        thumbs : les onglets de l'utilisateur
        export : export d'onglet
    """

    def __init__(self, arguments):
        log.debug("+ Initialisation de EnvoleProfileClient")
        try:
            self.opt_parse(arguments)
            self.make_server()
        except Exception:
            log.exception("Dans l'__init__ de EnvoleProfileClient")
            raise InitException()

    def user_datas_opt_parse(self, arguments):
        """
            parse des options contenant des attributs utilisateurs
        """
        self.user_datas = self.decode_user_datas(arguments[2])
        self.username = self.user_datas['user']
        if self.user_datas.has_key('details'):
            self.user_datas = self.user_datas['details']
        if self.user_datas.has_key('infos'):
            self.user_datas = self.user_datas['infos']
        log.info("  + Nom de l'utilisateur : %s" % self.username)
        log.debug("  + Données de l'utilisateur : %s" % self.user_datas)

    def thumb_exporter_opt_parse(self, arguments):
        """
            parser pour l'export d'onglet
        """
        (self.username, self.user_id, self.session_pt,
         self.thumb_id, self.thumb_name, self.thumb_indice) = arguments[2:]
        log.debug("  + User_id : %s" % self.user_id)
        log.debug("  + Thumb_id : %s" % self.thumb_id)
        log.debug("  + Thum_name : %s" %  self.thumb_name)
        log.debug("  + Thumb_indice : %s" % self.thumb_indice)
        log.debug("  + Session_PT : %s" % self.session_pt)

    def opt_parse(self, arguments):
        """
            parse les arguments passés à l'exécution du fichier
            récupère le mode et parse selon le mode
        """
        self.mode = arguments[1]
        log.info("  + On appelle l'action %s" % self.mode.upper())
        if self.mode == 'desktop':
            self.user_datas_opt_parse(arguments)
        elif self.mode == "thumbs":
            self.user_datas_opt_parse(arguments)
        elif self.mode == "exporter":
            self.thumb_exporter_opt_parse(arguments)
        else:
            self.user_datas_opt_parse(arguments)

    def make_server(self, ip=XMLRPC_IP_SERVER, port=XMLRPC_PORT):
        """
            instancie un client xmlrpc pour appeler le UserContext
        """
        log.debug("  * Client xmlrpclib : http://%s:%s/" % (ip, port))
        self.server = xmlrpclib.Server('http://%s:%s/' % (ip, port))

    def decode_user_datas(self, user_datas_str):
        """
            décode les données utilisateurs envoyées au format Json
        """
        try:
            return json.loads(user_datas_str)
        except Exception:
            log.exception("Le module json n'a pas pu décoder les informations \
utilisateurs")
            return {}

    def get_desktop_items(self):
        """
            appel les infos pour la mise en forme des
            boutons du bureau utilisateur
        """
        log.debug("  + Récupération des données pour les items de bureau")
        try:
            result = self.server.get_desktop_items(self.username,
                                                   self.user_datas)
        except Exception:
            log.exception("Échec au contact du serveur de profil")
            result = ""
        return_result(result)

    def init_thumbs(self):
        """
            appel l'initialisation des onglets utilisateurs dans la bdd
        """
        log.debug("  + Initialisation des onglets utilisateurs")
        try:
            initializer = InitThumbs(self.user_datas,
                                     self.username,
                                     self.server)
            initializer.insert_tabs_and_modules()
        except Exception:
            log.exception("Problème rencontré dans l'initialisation")

    def export_thumb(self):
        """
            exporte un onglet vers le gestionnaire de profil
        """
        log.debug("  + Exportation d'onglet vers le gestionnaire de profil")
        try:
            exporter = ExportThumbs(self.username,
                                    self.session_pt,
                                    self.user_id,
                                    self.thumb_id,
                                    self.thumb_name,
                                    self.thumb_indice,
                                    self.server,
                                    self.get_my_ip())
            return_result(exporter.export_thumb())
        except Exception:
            log.exception("Problème rencontré à l'export d'onglet")
            return_result("")

    def execute_action(self):
        """
            exécute l'action commandée
        """
        log.debug(" + Exécution de l'action : %s" % self.mode)
        request_keys = {'desktop':self.get_desktop_items,
                        'thumbs':self.init_thumbs,
                        'exporter':self.export_thumb}
        if self.mode == "desktop":
            self.get_desktop_items()
        elif self.mode == 'thumbs':
            self.init_thumbs()
        elif self.mode == "exporter":
            self.export_thumb()
        else:
            log.error("Unknown procedure Called : %s "% self.mode)

    def get_my_ip(self):
        return LOCAL_IP

def filter_sysargs(sysargs):
    """
        filtre les valeurs passées par ligne de commande
    """
    log.debug("Les arguments de l'appel système : %s" % str(sysargs))

def main():
    """
        Fonction d'appel du client
    """
    try:
        log.info("**** Appel du client ****")
        filter_sysargs(sys.argv)
        client = EnvoleProfilClient(sys.argv)
        client.execute_action()
        sys.exit(0)
    except InitException, error:
        log.exception("Exception à l'initialisation")
        print "<err>" + error.message + "</err>"
        sys.exit(1)
    except Exception:
        log.exception("Erreur dans la fonction main.")
        sys.exit(1)

if __name__ == '__main__':
    main()
