#-*-coding:utf-8-*-
"""
    Outil de validation des formulaires d'édition de base de données
"""
import xmlrpclib
import socket

from posh.backend import db_mapper
from posh.error import AddDBError, DelDBError, AddLinkDBError
from posh.error import DelLinkDBError, ModDBError
from posh.config.config import XMLRPC_IP_SERVER, XMLRPC_PORT
from posh.logger.logger import getLogger

import html_lib # import success_valid_msgs, echec_valid_msgs

log = getLogger('interface.data_handler.valid')

def reload_usercontext():
    """
        Reload le contexte utilisateur du serveur de profil
    """
    server = xmlrpclib.Server("http://%s:%s/"%(XMLRPC_IP_SERVER,XMLRPC_PORT))
    socket.setdefaulttimeout(10)
    server.db_reload()
    socket.setdefaulttimeout(None)


# TODO:
#    + suppression des anciens liens à l'ajout,
#    - message de retour
#    - contrôle des données

def add(_type, valid_dict):
    """ ajoute des éléments dans la table identifiée par _type
    """
    log.debug("** Demande de modification dans la table %s **" % (_type,))
    log.debug(" + Données pour la validation : %s" % valid_dict)
    message = {'message':[], 'alert':''}
    try:
        db_mapper.add(_type, valid_dict)
        message['message'].append(html_lib.valid_add_msg(_type, valid_dict))
        reload_usercontext()
    except AddDBError, error:
        log.exception("AddDBError")
        message['message'].append(html_lib.add_error_msg(error.table, error.element))
        message['alert'] = '_error'
    except Exception, msg:
        log.exception("Erreur inconnue")
        message['message'].append("Echec à l'ajout: <br> %s" % msg.message)
        message['alert'] = '_error'
    log.debug("** Message de retour : %s **" % message)
    return message

def set_links(_type, id1, id2s, items1, items2):
    """
        - définit les liens des id2s vers id1
            (en supprimant ceux qui ne doivent par y être)
        - construit les informations de retour depuis les libellés
            fournit pour la construction du formulaire
    """
    message = []
    alert = ""
    log.debug("** Ajout de lien entre des tables **")
    log.debug(" Nom de la table de jointure : %s" % _type)
    log.debug(" + id1 : %s" % id1)
    log.debug(" + id2s : %s\n" % id2s)
    log.debug(" + items1 : %s" % items1)
    log.debug(" + items2 : %s" % items1)
    try:
        added_id2s, deleted_id2s = db_mapper.set_links(_type, id1, id2s)

        # récupération du libellé de l'élément d'id id1
        libelle_id1 = html_lib.get_select_option_libelles(id1, items1)[0]

        # construction du message de retour pour les liens ajoutés
        if added_id2s:
            add_msg = [{'libelle':"Ajout de lien entre:", 'font':'bold'}]
            for id2 in added_id2s:
                libelle_id2 = html_lib.get_select_option_libelles(id2, items2)[0]
                add_msg.append({'libelle':"      %s et %s" % (libelle_id1, libelle_id2), 'tab':1})
        else:
            add_msg = [{'libelle':"Aucun lien n'a été ajouté", 'font':'bold'}]
        # construction du message de retour pour les liens supprimés
        if deleted_id2s:
            del_msg = [{'libelle':"Suppression de lien entre:", 'tab':0, 'font':'bold'}]
            for id2 in deleted_id2s:
                libelle_id2 = html_lib.get_select_option_libelles(id2, items2)[0]
                del_msg.append({'libelle':"      %s et %s" % (libelle_id1, libelle_id2), 'tab':1})
        else:
            del_msg = [{'libelle':"Aucun lien n'a été supprimé", 'font':'bold'}]
        message.extend(add_msg)
        message.extend(del_msg)

    except DelLinkDBError:
        log.exception("DelLinkDBError")
    except AddLinkDBError:
        log.exception("AddLinkDBError")

    except Exception, msg:
        log.exception("Echec à l'ajout de lien entre %s et %s \
%s" % (str(id1), str(id2s)))
        message.append({'libelle':"Echec à l'ajout de lien entre %s et %s pour le type %s : \
                %s" % (str(id1), str(id2s), _type, msg.message)})
        alert = "_error"

    try:
        reload_usercontext()
    except Exception, msg:
        log.exception("Échec du reload du contexte utilisateur")
    log.debug("** Message de retour : %s **" % message)
    return dict(message=message, alert=alert)

def del_(_type, ids, form_datas):
    log.debug("** Suppression d'un élément de la table %s **" % _type)
    log.debug(" - ids des éléments à supprimer : %s" % ids)
    log.debug(" \n - données de mise en forme founies (form_datas) : %s\n" % (form_datas,))
    message = []
    alert = ""
    try:
        db_mapper.del_items(_type, ids)
        form_datas = [f_data for f_data in form_datas if f_data['id'] in ids]
        message.extend(html_lib.valid_del_msg(_type, form_datas))

    except DelDBError, error:
        log.exception("Erreur à la suppression des éléments")
        form_data = [f_data for f_data in form_datas if f_data['id'] == error.elementid]
        message.append({'libelle':'Erreur:'})
        message.append(html_lib.del_error_msg(error.table, form_data))
        alert = "_error"

    except Exception:
        log.exception("Erreur inconnue à la suppression des éléments")
        message.append({'libelle':"Erreur inconnue à la suppression des éléments"})
        alert = "_error"
    try:
        reload_usercontext()
    except Exception:
        log.exception("Échec du reload du contexte utilisateur")
    log.debug("** Message de retour : %s **"  % message)
    return dict(message=message, alert=alert)

def modify(_type, item_id, value_dict, form_datas):
    """ valide la modification d'une entrée de la table
    """
    log.debug("** Modification d'un élément de la table %s **" % _type)
    log.debug(" + Id de l'élément à modifier %s" % item_id)
    log.debug(" + Dictionnaire des nouvelle valeur %s" % value_dict)
    log.debug(" \n  + données pour la mise en forme form_datas = %s\n" % form_datas)
    message = []
    alert = ""
    try:
        db_mapper.modify(_type, item_id, **value_dict)
        message.append({'libelle':'Modification affectée.', 'font':'bold'})
    except ModDBError, error:
        log.exception("Erreur à la modification de l'entrée de la table %s" % (_type,))
        for elem in form_datas:
            if elem['id'] == error.element_id:
                message.append(html_lib.mod_error_msg(error.table, elem))
                break
        alert = "_error"
    except:
        log.exception("Erreur inconnue à la modification de l'entrée de la table %s" % (_type,))
        message.append({"libelle":"Erreur à la modification de l'élément de la table %s" % _type})
        alert = "_error"
    try:
        reload_usercontext()
    except Exception:
        log.exception("Échec du reload du contexte utilisateur")
    log.debug("** Message de retour :%s **" % message)
    return dict(message=message, alert=alert)


