#-*-coding:utf-8-*-
""" pages spécifiques à l'application de gestion de profil posh
    Ces pages ne sont pas intégrées dans l'automtisation de l'interface
"""
from os.path import join, splitext, isdir, isfile
from os import listdir, unlink, system
import tempfile

from nevow.rend import Page
from nevow import loaders, tags as T, inevow
import Image

from interface.interface import tools
from interface.interface.config import HTML_DIR
from interface.interface.frags.message import Message
from posh.logger.logger import getLogger

log = getLogger("interface.frags.specific")

dest_dir = "/var/www/html/posh/includes/plugins/plugin_desktop/icones/download/"
dest_url = "%s/posh/includes/plugins/plugin_desktop/icones/download/"
temp_dir = "/tmp"
tempfile.tempdir = temp_dir

# largeur/hauteur d'une icone de bureau
ICONSIZE = (30,25)

accepted_exts = ('.jpg', '.png', '.ico', '.gif', '.JPG')

def test_extension(filename):
    """
    teste l'extension d'un fichier d'image (assertion)
    """
    assert splitext(filename)[-1] in accepted_exts

def is_icon(filename):
    """
        test l'extension d'un fichier
    """
    return splitext(filename)[-1] in accepted_exts

def writetempfile(filecontent):
    """ écrit le fichier image temporaire (avant redimensionnement)
    """
    f_name = tempfile.mktemp()
    # construction d'un nom pour le fichier temporaire (/tmp/....)
    while isfile(f_name):
        f_name = tempfile.mktemp()
    f_ = file(f_name, 'w')
    f_.write(filecontent)
    f_.close()
    return f_name

def resize_icon(filename, dest_file):
    """ redimensionne une icône """
    icon = Image.open(filename)
    new_ = icon.resize(ICONSIZE, Image.ANTIALIAS)
    new_.save(dest_file)

def del_temp_file(tempfilename):
    """ supprime le fichier temporaire """
    unlink(tempfilename)

def set_file_rights(filename):
    """ Autorise la lecture du fichier par www-data """
    system("/bin/chmod 644 '%s'" % filename)

def writefile(filename, filecontent):
    """ écrit le fichier image """
    log.debug("# Écriture du fichier image ")
    log.debug(" + Écriture du fichier temporaire")
    tempfilename = writetempfile(filecontent)
    log.debug(" + Redimensionnement de l'image et écriture de l'icône \
dans le répertoire approprié")
    resize_icon(tempfilename, filename)
    log.debug(" + Suppression du fichier temporaire")
    del_temp_file(tempfilename)
    set_file_rights(filename)
    return True

def get_icon_list(root_url):
    """
        renvoie la liste des icônes présentes dans le répertoire d'icônes
    """
    icons_url = dest_url % root_url
    return [(icon, join(icons_url , icon))
            for icon in listdir(dest_dir) if is_icon(icon)]

def del_icon(f_name):
    """
        supprime une icône
    """
    filepath = join(dest_dir, f_name)
    log.debug("# Suppression du fichier %s" % (filepath,))
    if isfile(filepath):
        unlink(filepath)
        log.debug(" + Le fichier a bien été supprimé")
        return {'libelle': "Le fichier %s a bien été supprimé" % f_name}
    else:
        log.debug(" - Erreur : le fichier n'existe pas.")
        return {'libelle' : "Erreur : le fichier %s n'existe pas." % (
                                            f_name,), "alert":"_error"}


class ValidDel(Message):
    """
        page de validation de suppression d'une icone téléchargée
    """
    isLeaf = True
    close_btn_code = "Refresh('icon_upload');setContent('popup_container', '')"

    def locateChild(self, ctx, segments):
        return self, ()

    def data_message(self, ctx, data):
        """ valide la suppression d'élément
        """
        request = inevow.IRequest(ctx)
        if request.args.has_key('todel'):
            todel = request.args['todel'][0]
            return dict(message=[del_icon(todel)],alert="")
        return [{'message':"Il manque des paramètres pour la suppression",
                                                         'alert':'_error'}]

class ValidUpload(Message):
    """
        page de validation de l'upload d'une image
    """
    close_btn_code = "Refresh('icon_upload');"

    def locateChild(self, ctx, segments):
        return self, ()

    def data_message(self, ctx, data):
        """ renvoie le message de retour de téléchargement de fichier
        """
        request = inevow.IRequest(ctx)

        log.info("** Téléchargement d'une icone de bureau **")

        if not request.args.has_key('icon_filename') or \
                    not request.args.has_key('uploadfile'):
            log.error("---Erreur : pas de fichier téléchargé")
            message = [{'libelle':"Erreur: pas de fichier téléchargé"}]
            return dict(message=message, alert="_error")

        log.debug("+ Récuperation des valeurs dans la requête")
        if request.args.has_key('uploadfile'):
            filecontent = request.args['uploadfile'][0]
            if request.args.has_key('icon_filename'):
                filename = request.args['icon_filename'][0].split('/')[-1]
                filename = filename.split('\\')[-1]

        log.debug("+ Test du contenu")
        if not filecontent:
            log.error(" - Erreur : fichier vide.")
            message = [{'libelle':"Erreur: fichier vide \
(pas de fichier téléchargé ? )"}]
            return dict(message=message, alert="_error")

        try:
            log.debug("+ Test de l'extension de %s " % filename)
            test_extension(filename)
        except AssertionError:
            log.error(" - Erreur: type de fichier invalide")
            message = [{'libelle':'Erreur: mauvais type de fichier'}]
            return {'message':message, 'alert':'_error'}
        if isdir(dest_dir):
            if not isfile(join(dest_dir, filename)):
                if writefile(join(dest_dir, filename), filecontent):
                    root_url_without_port = tools._get_root_url_without_port(
                                                                          ctx)
                    access_url = join(dest_url, filename) % (
                                                     root_url_without_port,)
                    message = [{'libelle':"L'image a bien été téléchargée, \
l'url d'accès est : " }]
                    js_link = "javascript:fenetre('%s', '%s')" % (access_url,
                                                                  filename)
                    message.append(dict(libelle=T.a(href=js_link)[access_url],
                                         font='bold'))
                    log.debug("** Fin du téléchargement **")
                    return dict(message=message, alert="")
                else:
                    log.error(" - Erreur à l'écriture de l'image")
                    message = [{'libelle':"Erreur à l'écriture de l'image",
                                "font":"bold"}]
                    return dict(message=message, alert="_error")
            else:
                log.error(" - Erreur : un fichier du même nom a déjà \
été téléchargé")
                message = [{'libelle':"Erreur : un fichier du même nom \
a déjà été téléchargé", 'font':'bold'}]
                return dict(message=message, alert="_error")
        log.error(" - Erreur : le répertoire de destination \
%s n'existe pas." % (dest_dir,))
        return dict(message=["Erreur, le répertoire de destination %s \
n'existe pas." % dest_dir], alert="_error")


class IconeUpload(Page):
    docFactory = loaders.xmlfile(join(HTML_DIR, 'iconeupload.xml'))
    children = {'valid':ValidUpload(),
                'delete':ValidDel(),
                }
    myurl = ['icone']
    valid_url = ['icone', 'valid']
    input_id = "img_file"
    delurl = ['icone', 'delete', '?todel=%s']

    def locateChild(self, ctx, segments):
        request = inevow.IRequest(ctx)
        child, remainingSegments = Page.locateChild(self, ctx, segments)
        if child:
            return child, remainingSegments
        return self, ()

    def render_attr_slots(self, ctx, data):
        """rend les attributs remplissant les 'slots' du template
        """
        ctx.fillSlots('valid_url', join(tools._get_root_url(ctx),
                                        *self.valid_url))
        ctx.fillSlots('input_name', self.input_id)
        myurl = join(tools._get_root_url(ctx), *self.myurl)
        ctx.fillSlots('url', myurl)
        return ctx.tag

    def render_download_input(self, ctx, data):
        """ rend l'input de téléchargement
        """
        return [T.td[T.label["Fichier de type image( %s )" % (
                                                ' ,*'.join(accepted_exts))]],
                T.td[T.input(_type='file',
                             name='uploadfile',
                             _id=self.input_id,
                             onchange="$('filename').value=this.value;"),]
                ]

    def render_filename_input(self, ctx, data):
        """ rend l'input de nom de fichier
        """
        return [T.td[T.label["Nouveau nom (le même par défaut):"]],
                T.td[T.input(_type="text",
                             name="icon_filename",
                             _id="icon_filename")],
               ]

    def data_iconlist_header(self, ctx, data):
        return ['Nom du fichier',
                "Url du fichier (pour utiliser l'icône dans le gestionnaire \
de bureau)",
                "Voir",
                '']

    def render_iconlist_header(self, ctx, data):
        """
            renvoie la description pour l'entête
            de la liste des icônes à gérer
        """
        for value in data:
            yield T.td()[value]

    def data_iconlist_rows(self, ctx, data):
        """
            Ligne de la liste d'icônes
        """
        root_url = tools._get_root_url_without_port(ctx)
        return get_icon_list(root_url)

    def render_iconlist_rows(self, ctx, data):
        """
            propose la liste des icônes disponibles sur le serveur
        """
        for icon in data:
            yield T.tr[T.td[icon[0]],
                       T.td[self.truncate_link(icon[1])],
                       T.td[
                        T.a(href="javascript:fenetre('%s', '%s')" % (
                                    icon[1], icon[0]))["Voir l'icône"]],
                       T.td[ self._suppr_btn(ctx, icon[0])]]

    def truncate_link(self, link):
        """
            Tronque un lien
        """
        return '/' + '/'.join(link.split('/')[3:])

    def _suppr_btn(self, ctx, icon_name):
        """
            renvoie le bouton de suppression d'un élément
        """
        url = join( tools._get_root_url(ctx), *self.delurl ) % (icon_name)
        js_link = "javascript:callUrl('%s', [], \
'iconuploadmsgcontainer')" % url
        return   T.a(href=js_link)[T.div[
                            "Suppr",
                            T.img(src='/image/supprimer.gif')
                        ]]
