#-*-coding:utf-8-*-
"""
    Serveur xmlrpc renvoyant des données de profil
    - Ecriture des onglets obligatoires dans la base
    - Récupération des descriptions des boutons du bureau utilisateur
"""
from os.path import isfile
from xmlrpclib import Server

from twisted.internet import reactor
from twisted.web import server
from twisted.internet import defer
from twisted.web.xmlrpc import XMLRPC

from posh.perm.permission import USERCONTEXT
from posh.backend.db_mapper import add_thumb, get_thumb, get_keys
from posh.config.config import XMLRPC_PORT as PORT
from posh.config.config import AUTH_SERVER, SERVICE_AUTH_NAME
from posh.logger.logger import getLogger

log = getLogger('profile.xmlrpc.server')

ALLOWED_USERS = ['admin']
ISTEST = False
XML_HEADER = """<?xml version='1.0' encoding='utf-8' ?>"""

BUTTONDATAS_TEMPLATE = """<buttondatas>
%s
</buttondatas>"""

# les 2 fonctions qui suivent ont été dupliquées
# depuis zephir.lib_zephir
CHARSET = 'UTF-8'

def convert(objet):
    """Transforme les objets unicode contenus dans un objet en chaines
    """
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == dict:
        dico = {}
        for cle in objet.keys():
            dico[cle] = convert(objet[cle])
        return dico
    if type(objet) == unicode:
        string =  objet.encode(CHARSET)
        return string
    return objet

def u(objet):
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(u(item))
        return l
    if type(objet) == dict:
        dico = {}
        for cle in objet.keys():
            dico[cle] = u(objet[cle])
        return dico
    if type(objet) == str:
        string = unicode(objet, CHARSET)
        return string
    return objet

def reload_context(context):
    """
        Reload le contexte utilisateur dans le gestionnaire de profil
    """
    context.reload()
    return "<success>Yes</success>"

def err_reload(err):
    """
        Erreur au reload
    """
    log.error(err)
    return "<error>No</error>"

def test_pt(proxy_ticket):
    """
        test un pt auprès du serveur d'authentification
    """
    auth_server = Server(AUTH_SERVER)
    return auth_server.verify_app_ticket(proxy_ticket,
                                      SERVICE_AUTH_NAME)[0] or ISTEST

def error_tag(message):
    """
        Formatte le message sous forme d'erreur
    """
    log.error("--> On renvoie : %s" % (message,))
    return "<err>" + str(message) + "</err>"

def msg_tag(message):
    """
        Formatte le message sous forme de message de retour
    """
    log.debug("--> On renvoie : %s" % (message,))
    return "<msg>" + str(message) + "</msg>"

class UserContextServer(XMLRPC):
    """ Serveur Xmlrpc du gestionnaire de profil
        renvoie les decriptions des items de bureau
        et des onglets pour un utilisateur
    """

    def __init__(self):
        XMLRPC.__init__(self)
        self.usercontext = USERCONTEXT

    def xmlrpc_db_reload(self):
        """
            Appel du reload de la base
        """
        log.info("#### Appel du reload de la base ")
        _deferred = defer.Deferred()
        _deferred.callback(self.usercontext)
        _deferred.addCallback(reload_context)
        _deferred.addErrback(err_reload)
        return _deferred

    def xmlrpc_get_desktop_items(self, username, user_datas):
        """ renvoie le xml décrivant les onglets de l'utilisateur
        """
        log.info("#### Appel des items de bureau pour : %s" % username)
        log.debug("++++   Les infos additionnelles : %s " %user_datas)
        user = self.usercontext.connect_user(username, user_datas)

        log.debug("---->  Le retour xml : %s " % ''.join([desktop_item.toxml()
             for desktop_item in self.usercontext.get_desktop_items(user)]))

        buttons = BUTTONDATAS_TEMPLATE % ''.join([desktop_item.toxml()
              for desktop_item in self.usercontext.get_desktop_items(user)])
        buttuns = u(buttons)
        return XML_HEADER \
               + '<main>' \
               + buttons \
               + '</main>'

    def xmlrpc_get_thumbs(self, username, user_datas):
        """
            récupère les onglets de l'utilisateur
        """
        log.info("#### Appel des onglets pour : %s " % username)
        user = self.usercontext.connect_user(username, user_datas)
        # on rajoute le nom dans la description de l'onglet
        thumbs = []
        for thumb in self.usercontext.get_thumbs(user):
            thu = u(thumb.to_dbfield())
            log.debug("---->    On renvoie l'onglet : %s" %thumb.name)
            thumbs.append(thu)
        return thumbs

    def xmlrpc_export_thumb(self, username, session_pt, client_ip,
                                    thumb_name, thumb_description):
        """
            rajoute un onglet dans la base
        """
        username = convert(username)
        session_pt = convert(session_pt)
        client_ip = convert(client_ip)
        thumb_name = convert(thumb_name)
        thumb_description = convert(thumb_description)

        log.info("# Insertion d'onglet dans le gestionnaire de profil pour \
l'utilisateur : %s" % (username,))
        log.debug(" + ip du portail appelant : %s" % client_ip)
        log.debug(" + description de l'onglet : %s" % (thumb_description,))

        if test_pt(session_pt):
            if username in ALLOWED_USERS:
                keys = get_keys('thumb')
                thumb_description = thumb_description[:6]
                thumb = dict([(key, str(thumb_description[i]))
                                for i,key in enumerate(keys)])

                # hack pour ne pas ecrire le admin_profile_id
                # si la page a une url externe
                # cette valeur est discriminante et permet d'identifier
                # les pages avec widgets
                if thumb['poshtype'] == '2':
                    thumb['admin_profile_id'] = '-1'
                if get_thumb(name=thumb['name']):
                    message = error_tag("l'onglet %s existe déjà \
dans le gestionnaire de profil." % (thumb['name'],))
                    return message
                try:
                    add_thumb(thumb)
                    return msg_tag("L'onglet %s a bien été ajouté dans le \
gestionnaire de profil." % (thumb_name,))
                except Exception, msg:
                    log.exception("Ecriture de l'onglet %s" % thumb_name)
                    return error_tag("l'écriture de l'onglet %s dans la base \
a échouée</err>" % (thumb_name,))
        log.error("WARNING : ******** ACCES ILLEGAL")
        return error_tag("vous n'êtes pas autorisé à effectuer cette action")

def main():
    """
        lance le serveur de profil
    """
    context_server = UserContextServer()
    reactor.listenTCP(PORT, server.Site(context_server))
    reactor.run()

if __name__ == '__main__':
    main()
